/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM /builds/slave/cen-osx64-xr/build/dom/interfaces/base/domstubs.idl
 */

#ifndef __gen_domstubs_h__
#define __gen_domstubs_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
typedef PRUint64  DOMTimeStamp;

class nsIDOMAttr; /* forward declaration */

class nsIDOMCDATASection; /* forward declaration */

class nsIDOMCharacterData; /* forward declaration */

class nsIDOMComment; /* forward declaration */

class nsIDOMDOMImplementation; /* forward declaration */

class nsIDOMDocument; /* forward declaration */

class nsIDOMDocumentFragment; /* forward declaration */

class nsIDOMDocumentType; /* forward declaration */

class nsIDOMElement; /* forward declaration */

class nsIDOMEntity; /* forward declaration */

class nsIDOMEntityReference; /* forward declaration */

class nsIDOMNSDocument; /* forward declaration */

class nsIDOMNSElement; /* forward declaration */

class nsIDOMNamedNodeMap; /* forward declaration */

class nsIDOMNode; /* forward declaration */

class nsIDOMNodeList; /* forward declaration */

class nsIDOMNotation; /* forward declaration */

class nsIDOMProcessingInstruction; /* forward declaration */

class nsIDOMText; /* forward declaration */

class nsIDOMDOMStringList; /* forward declaration */

class nsIDOMDOMTokenList; /* forward declaration */

class nsIDOMNameList; /* forward declaration */

class nsIDOMClientRect; /* forward declaration */

class nsIDOMClientRectList; /* forward declaration */

class DOMException; /* forward declaration */

class RangeException; /* forward declaration */

class nsIDOMStyleSheetList; /* forward declaration */

class nsIDOMLinkStyle; /* forward declaration */

class nsIDOMStyleSheet; /* forward declaration */

class nsIDOMMediaList; /* forward declaration */

class nsIDOMAbstractView; /* forward declaration */

class nsIDOMDocumentView; /* forward declaration */

class nsIDOMWindow; /* forward declaration */

class nsIDOMWindowInternal; /* forward declaration */

class nsIDOMWindowCollection; /* forward declaration */

class nsIDOMPlugin; /* forward declaration */

class nsIDOMPluginArray; /* forward declaration */

class nsIDOMMimeType; /* forward declaration */

class nsIDOMMimeTypeArray; /* forward declaration */

class nsIDOMBarProp; /* forward declaration */

class nsIDOMNavigator; /* forward declaration */

class nsIDOMScreen; /* forward declaration */

class nsIDOMHistory; /* forward declaration */

class nsIDOMEvent; /* forward declaration */

class nsIDOMEventTarget; /* forward declaration */

class nsIDOMEventListener; /* forward declaration */

class nsIDOMEventGroup; /* forward declaration */

class nsIDOMHTMLElement; /* forward declaration */

class nsIDOMHTMLFormElement; /* forward declaration */

class nsIDOMHTMLCollection; /* forward declaration */

class nsIDOMHTMLHeadElement; /* forward declaration */

class nsIDOMCSSValue; /* forward declaration */

class nsIDOMCSSValueList; /* forward declaration */

class nsIDOMCSSPrimitiveValue; /* forward declaration */

class nsIDOMCSSRule; /* forward declaration */

class nsIDOMCSSRuleList; /* forward declaration */

class nsIDOMCSSStyleSheet; /* forward declaration */

class nsIDOMCSSStyleDeclaration; /* forward declaration */

class nsIDOMCounter; /* forward declaration */

class nsIDOMRect; /* forward declaration */

class nsIDOMRGBColor; /* forward declaration */

class nsIDOMCSSStyleRule; /* forward declaration */

class nsIDOMCSSStyleRuleCollection; /* forward declaration */

class nsIDOMHTMLTableCaptionElement; /* forward declaration */

class nsIDOMHTMLTableSectionElement; /* forward declaration */

class nsIDOMRange; /* forward declaration */

class nsIDOMCRMFObject; /* forward declaration */

class nsIDOMCrypto; /* forward declaration */

class nsIDOMPkcs11; /* forward declaration */


#endif /* __gen_domstubs_h__ */
